/*****************************************************************************
**																			**
**	 Name: 	UART  Software Interface										**	
**																			**
******************************************************************************

(C) Copyright 2003 - Analog Devices, Inc.  All rights reserved.

File Name:		uartlib.asm

Date Modified:	04/03/03		BK		Rev 1.0

Software:       GNU DCC 4.1+

Purpose:		This file contains some library functions
                commonly used on UART processing.				

05/22/07 Ronetix, ported from ASM to C

******************************************************************************/
#include "../inc/cdefBF532.h"

/*****************************************************************************
 *
 *  Assuming 8 data bits, this functions expects a '@' (ASCII 0x40) character 
 *  on the UART RX pin. 
 *  Timer 2 performs the autobaud detection. Also special support for
 *  half-duplex systems is provided. 
 *
 ****************************************************************************/
int uart_autobaud()
{
	int tt;
	
/*****************************************************************************
 *
 *  First of all, enable UART clock. It is required for autobaud detection on 
 *  silicon revision 0.0.
 *
 ****************************************************************************/ 
	*pUART_GCTL = UCEN;

/*****************************************************************************
 *
 *  Activate Loopback mode in order the receive channel is disconnected 
 *  from RX pin during autobaud detection.
 *
 ****************************************************************************/ 
	*pUART_MCR = LOOP_ENA;
	
/*****************************************************************************
 *
 *  Setup Timer 2 Controller to do the autobaud detection. Timer captures
 *  duration between two falling edges. It expects a '@' (ASCII 0x40) 
 *  character. 8-bit, no parity assumed.
 *
 ****************************************************************************/ 
/*****************************************************************************
 *
 *  Disable Timer 2 first, in case there was an unexpected history.  
 *
 ****************************************************************************/ 
	*pTIMER_DISABLE = TIMDIS2;
	*pTIMER_STATUS 	= TRUN2 | TOVL_ERR2 | TIMIL2;

/*****************************************************************************
 *
 *  Capture from UART RxD pin. Select period capture from falling edge to 
 *  falling edge. Enable IRQ_ENA, but don't enable the interrupt at system 
 *  level (SIC). 
 *
 ****************************************************************************/ 
	*pTIMER2_CONFIG = TIN_SEL | IRQ_ENA | PERIOD_CNT | WDTH_CAP;

/*****************************************************************************
 *
 *  Start the timer and wait until the according interrupt latch bit TIMIL2
 *  in the TIMER_STATUS register is set. Then, two falling edges on the RxD
 *  pin have been detected.
 *
 ****************************************************************************/ 
	*pTIMER_ENABLE 	= TIMEN2;
	
	while ( ~*pTIMER_STATUS & TIMIL2 );

/*****************************************************************************
 *
 *  Disable Timer 2 again  
 *
 ****************************************************************************/ 
	*pTIMER_DISABLE = TIMDIS2;
	*pTIMER_STATUS 	= TRUN2 | TOVL_ERR2 | TIMIL2;
	
/*****************************************************************************
 *
 *  Save period value to tt
 *
 ****************************************************************************/ 
	tt = *pTIMER2_PERIOD;
	
/*****************************************************************************
 *
 *  In order to support also half-duplex connections, we need to delay any 
 *  transmission, in order the sent character does not overlap the autobaud
 *  pattern.
 *
 *  Use Timer 2 to perform this delay. Note that the Period Register still
 *  contains the proper value and the Width Register is not used.
 *
 ****************************************************************************/	
	*pTIMER2_CONFIG = OUT_DIS | IRQ_ENA | PERIOD_CNT | PWM_OUT;
	*pTIMER_ENABLE 	= TIMEN2;

	while ( ~*pTIMER_STATUS & TIMIL2 );

/*****************************************************************************
 *
 *  Disable Timer 2 again  
 *
 ****************************************************************************/ 
	*pTIMER_DISABLE = TIMDIS2;
	*pTIMER_STATUS 	= TRUN2 | TOVL_ERR2 | TIMIL2;
	
/*****************************************************************************
 *
 *  Deactive Loopback mode again
 *
 ****************************************************************************/ 	
	*pUART_MCR = 0;
	
/*****************************************************************************
 *
 *  done !
 *
 ****************************************************************************/ 	
	return tt;
}

/*****************************************************************************
 *
 *  Configures UART in 8 data bits, no parity, 1 stop bit mode.
 *
 ****************************************************************************/
void uart_init( int dd )
{
/*****************************************************************************
 *
 *  First of all, enable UART clock. 
 *
 ****************************************************************************/ 
	*pUART_GCTL = UCEN;

/*****************************************************************************
 *
 *  Read period value and apply formula:  DL = PERIOD / 16 / 8
 *  Write result to the two 8-bit DL registers (DLH:DLL).
 *
 ****************************************************************************/ 
	*pUART_LCR 	= DLAB;
	*pUART_DLL 	= dd;
	*pUART_DLH 	= dd>>8;

/*****************************************************************************
 *
 *  Clear DLAB again and set UART frame to 8 bits, no parity, 1 stop bit. 
 *  This may differ in other scenarious.
 *
 ****************************************************************************/ 
	*pUART_LCR 	= WLS(8);
}

/*****************************************************************************
 *
 *	This function polls the TEMT bit in the LSR register and waits until
 *  all data has been shifted out.
 *
 ****************************************************************************/
void uart_wait4temt()
{
	while ( ~*pUART_LSR & TEMT );
}

/*****************************************************************************
 *
 *	This function disables the UART clock, but polls the TEMT bit before
 *
 ****************************************************************************/
void uart_disable()
{
	uart_wait4temt();
	*pUART_GCTL = 0;
}

/*****************************************************************************
 *
 *	This function transmits a character by polling THRE bit in the LSR
 *  register.
 *
 ****************************************************************************/
void uart_putc( int cc )
{
	while ( ~*pUART_LSR & THRE );
	*pUART_THR = cc;
}

/*****************************************************************************
 *
 *	This function transmits a byte string by polling THRE bit in the LSR
 *  register. The string must be NULL-terminated (C-style).
 *
 ****************************************************************************/
void uart_puts( char *ss )
{
	while ( *ss )
		uart_putc( *ss++ );
}

/*****************************************************************************
 *
 *	This function transmits the content of the r0 register in hexdecimal
 *  representation. This is based on polling operation.
 *
 ****************************************************************************/
void uart_putreg( int rr )
{
	int ii;
	uart_puts( " 0x" );
	
	for ( ii=28; ii>=0; ii-=4 )
		if ( ((rr>>ii) & 0xF) <= 9 )
			uart_putc( ((rr>>ii) & 0xF) + '0' );
		else
			uart_putc( ((rr>>ii) & 0xF) + 'A' - 10 );
}
